@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,22 +15,21 @@
 
 import math
 import traceback
-from typing import Any, Union
+from typing import NoReturn, Union
 
 from hypothesis import Verbosity, settings
 from hypothesis.errors import CleanupFailed, InvalidArgument, UnsatisfiedAssumption
 from hypothesis.internal.conjecture.data import ConjectureData
-from hypothesis.internal.reflection import deprecated_posargs
 from hypothesis.internal.validation import check_type
 from hypothesis.reporting import report, verbose_report
 from hypothesis.utils.dynamicvariables import DynamicVariable
 
 
-def reject():
+def reject() -> NoReturn:
     raise UnsatisfiedAssumption()
 
 
-def assume(condition: Any) -> bool:
+def assume(condition: object) -> bool:
     """Calling ``assume`` is like an :ref:`assert <python:assert>` that marks
     the example as bad, rather than failing the test.
 
@@ -45,6 +44,18 @@ def assume(condition: Any) -> bool:
 _current_build_context = DynamicVariable(None)
 
 
+def currently_in_test_context() -> bool:
+    """Return ``True`` if the calling code is currently running inside an
+    :func:`@given <hypothesis.given>` or :doc:`stateful <stateful>` test,
+    ``False`` otherwise.
+
+    This is useful for third-party integrations and assertion helpers which
+    may be called from traditional or property-based tests, but can only use
+    :func:`~hypothesis.assume` or :func:`~hypothesis.target` in the latter case.
+    """
+    return _current_build_context.value is not None
+
+
 def current_build_context():
     context = _current_build_context.value
     if context is None:
@@ -124,7 +135,6 @@ def event(value: str) -> None:
     context.data.note_event(value)
 
 
-@deprecated_posargs
 def target(observation: Union[int, float], *, label: str = "") -> None:
     """Calling this function with an ``int`` or ``float`` observation gives it feedback
     with which to guide our search for inputs that will cause an error, in
