@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -282,3 +282,46 @@ def pop_random(random, seq):
     i = random.randrange(0, len(seq))
     swap(seq, i, len(seq) - 1)
     return seq.pop()
+
+
+class NotFound(Exception):
+    pass
+
+
+class SelfOrganisingList:
+    """A self-organising list with the move-to-front heuristic.
+
+    A self-organising list is a collection which we want to retrieve items
+    that satisfy some predicate from. There is no faster way to do this than
+    a linear scan (as the predicates may be arbitrary), but the performance
+    of a linear scan can vary dramatically - if we happen to find a good item
+    on the first try it's O(1) after all. The idea of a self-organising list is
+    to reorder the list to try to get lucky this way as often as possible.
+
+    There are various heuristics we could use for this, and it's not clear
+    which are best. We use the simplest, which is that every time we find
+    an item we move it to the "front" (actually the back in our implementation
+    because we iterate in reverse) of the list.
+
+    """
+
+    def __init__(self, values=()):
+        self.__values = list(values)
+
+    def __repr__(self):
+        return "SelfOrganisingList(%r)" % (self.__values,)
+
+    def add(self, value):
+        """Add a value to this list."""
+        self.__values.append(value)
+
+    def find(self, condition):
+        """Returns some value in this list such that ``condition(value)`` is
+        True. If no such value exists raises ``NotFound``."""
+        for i in range(len(self.__values) - 1, -1, -1):
+            value = self.__values[i]
+            if condition(value):
+                del self.__values[i]
+                self.__values.append(value)
+                return value
+        raise NotFound("No values satisfying condition")
